(*-----------------------------------------------------------------------*)
(* Unit MATLIB : mathematical functions and subroutines                  *)
(*-----------------------------------------------------------------------*)


{$N+,E+}       {Coprozessor verwenden, wenn nicht da emulieren !}
{$G-}          {Code erzeugen fuer 8086 CPU}


UNIT MATLIB;

INTERFACE

  (* vectors and matrices *)

  TYPE INDEX  = (X,Y,Z);
       VECTOR = ARRAY[INDEX] OF extended;
       extended3  = ARRAY[1.. 3] OF extended;
       extended33 = ARRAY[1.. 3] OF extended3;
       MAT3X  = ARRAY[1.. 3] OF VECTOR;

  (* Chebyshev polynomials *)

  CONST MAX_TP_DEG  = 13;                        (* maximum degree       *)
  TYPE  TPOLYNOM = RECORD                        (* Chebyshev polynomial *)
                     M  : INTEGER;                       (* degree       *)
                     A,B: extended;                          (* interval     *)
                     C  : ARRAY [0..MAX_TP_DEG] OF extended; (* coefficients *)
                   END;

  (* Vector and matrix for least squares systems:                    *)
  (* minimum dimensions for use with program FOTO are given below;   *)
  (* they may be increased as needed by the user.                    *)

  TYPE  LSQVEC =  ARRAY[1..3] OF extended;
        LSQMAT =  ARRAY[1..30,1..5] OF extended;


  (* functions and procedures *)

  FUNCTION  ACS    ( X: extended ): extended;
  FUNCTION  ASN    ( X: extended ): extended;
  FUNCTION  ATN    ( X: extended ): extended;
  FUNCTION  ATN2   ( Y,X: extended ): extended;
  PROCEDURE CART   ( R,THETA,PHI: extended; VAR X,Y,Z: extended );
  PROCEDURE CROSS  ( A,B: VECTOR; VAR C: VECTOR );
  FUNCTION  CS     ( X: extended ): extended;
  FUNCTION  CUBR   ( X: extended ): extended;
  FUNCTION  DOT    ( A,B: VECTOR ): extended;
  PROCEDURE DDD    ( D,M: INTEGER;S: extended;VAR DD: extended );
  PROCEDURE DMS    ( DD: extended;VAR D,M: INTEGER;VAR S: extended );
  PROCEDURE LSQFIT ( A: LSQMAT; N,M: INTEGER; VAR S: LSQVEC );
  FUNCTION  NORM   ( A: VECTOR ): extended;
  PROCEDURE POLAR  ( X,Y,Z: extended; VAR R,THETA,PHI: extended );
  PROCEDURE QUAD   ( Y_MINUS,Y_0,Y_PLUS:  extended;
                     VAR XE,YE,ZERO1,ZERO2: extended; VAR NZ: INTEGER );
  FUNCTION  SN     ( X: extended ): extended;
  FUNCTION  T_EVAL ( F:  TPOLYNOM; X:  extended ):  extended;
  FUNCTION  TN     ( X: extended ): extended;

IMPLEMENTATION

  (*$I \MATH\MATH.PAS    *)

END.

