(*----------------------------------------------------------------------*)
(* CALDAT: Finds the civil calendar date for a given value              *)
(*         of the Modified Julian Date (MJD).                           *)
(*         Julian calendar is used up to 1582 October 4,                *)
(*         Gregorian calendar is used from 1582 October 15 onwards.     *)
(*----------------------------------------------------------------------*)
PROCEDURE CALDAT(MJD:extended; VAR DAY,MONTH,YEAR:INTEGER;VAR HOUR:extended);
  VAR B,D,F     : INTEGER;
      JD,JD0,C,E: extended;
  BEGIN
    JD  := MJD + 2400000.5;
    (* JD0 := TRUNC(JD+0.5);      *)    (* Standard Pascal  *)
       JD0 := INT(JD+0.5);              (* TURBO Pascal     *)
    (* JD0 := LONG_TRUNC(JD+0.5); *)    (* ST Pascal plus   *)
    IF (JD0<2299161.0)                            (* calendar:    *)
      THEN BEGIN B:=0; C:=JD0+1524.0 END          (* -> Julian    *)
      ELSE BEGIN                                  (* -> Gregorian *)
             B:=TRUNC((JD0-1867216.25)/36524.25);
             C:=JD0+(B-TRUNC(B/4))+1525.0
           END;
    D    := TRUNC((C-122.1)/365.25);          E     := 365.0*D+TRUNC(D/4);
    F    := TRUNC((C-E)/30.6001);
    DAY  := TRUNC(C-E+0.5)-TRUNC(30.6001*F);  MONTH := F-1-12*TRUNC(F/14);
    YEAR := D-4715-TRUNC((7+MONTH)/10);       HOUR  := 24.0*(JD+0.5-JD0);
  END;

(*-----------------------------------------------------------------------*)
(* ETMINUT: Difference of Ephemeris Time and Universal Time              *)
(*          (polynomial approximation valid from 1900 to 1985)           *)
(*          T:     time in Julian centuries since J2000                  *)
(*                 ( = (JD-2451545.0)/36525.0 )                          *)
(*          DTSEC: DT=ET-UT in sec (only if VALID=TRUE)                  *)
(*          VALID: TRUE for times between 1900 and 1985, FALSE otherwise *)
(*-----------------------------------------------------------------------*)
PROCEDURE ETMINUT(T: extended; VAR DTSEC: extended; VAR VALID: BOOLEAN);
  BEGIN
    VALID := ( (-1.0<T) AND (T<-0.15) );
    IF (VALID) THEN
      BEGIN
        DTSEC := ((((-339.84*T-516.52)*T-160.22)*T)+92.23)*T+71.28;
      END;
  END;
(*-----------------------------------------------------------------------*)
(* LMST: local mean sideextended time                                        *)
(*-----------------------------------------------------------------------*)
FUNCTION LMST(MJD,LAMBDA:extended):extended;
  VAR MJD0,T,UT,GMST: extended;
  FUNCTION FRAC(X:extended):extended;
    BEGIN  X:=X-TRUNC(X); IF (X<0) THEN X:=X+1; FRAC:=X  END;
  BEGIN
    (* MJD0:=TRUNC(MJD); *)       (* Standard Pascal  *)
    MJD0:=INT(MJD);               (* TURBO Pascal     *)
    (* MJD0:=LONG_TRUNC(MJD); *)  (* ST Pascal plus   *)
    UT:=(MJD-MJD0)*24; T:=(MJD0-51544.5)/36525.0;
    GMST:=6.697374558 + 1.0027379093*UT
            +(8640184.812866+(0.093104-6.2E-6*T)*T)*T/3600.0;
    LMST:=24.0*FRAC( (GMST-LAMBDA/15.0) / 24.0 );
  END;

(*-----------------------------------------------------------------------*)
(* MJD: Modified Julian Date                                             *)
(*      The routine is valid for any date since 4713 BC.                 *)
(*      Julian calendar is used up to 1582 October 4,                    *)
(*      Gregorian calendar is used from 1582 October 15 onwards.         *)
(*-----------------------------------------------------------------------*)
FUNCTION MJD(DAY,MONTH,YEAR:INTEGER;HOUR:extended):extended;
  VAR A: extended; B: INTEGER;
  BEGIN
    A:=10000.0*YEAR+100.0*MONTH+DAY;
    IF (MONTH<=2) THEN BEGIN MONTH:=MONTH+12; YEAR:=YEAR-1 END;
    IF (A<=15821004.1)
      THEN B:=-2+TRUNC((YEAR+4716)/4)-1179
      ELSE B:=TRUNC(YEAR/400)-TRUNC(YEAR/100)+TRUNC(YEAR/4);
    A:=365.0*YEAR-679004.0;
    MJD:=A+B+TRUNC(30.6001*(MONTH+1))+DAY+HOUR/24.0;
  END;
